#include "qimagewidget.h"
#include <QMouseEvent>
#include <QPainter>
#include <QFileDialog>

QImageWidget::QImageWidget(QWidget *parent) : QWidget(parent)
{
  //значения параметров виджета по умолчанию
  _noImageMessage = "NO IMAGE";
  _backgroundColor = Qt::white;
  _margin = 4;

  //вызвапть слот перерисовки при изменении размера рамки
  connect(this, SIGNAL(marginChanged()), SLOT(repaint()));
  //вызвать слот перерисовки при изменении картинки
  connect(this, SIGNAL(pixmapChanged()), SLOT(repaint()));
}

QImageWidget::~QImageWidget()
{

}

QRect QImageWidget::actualImageRect()
{
  QRect imageRect = rect();
  imageRect.setSize(_originalImage.size().scaled(size(), Qt::KeepAspectRatio));
  imageRect.moveCenter(rect().center());
  return imageRect;
}

void QImageWidget::updateMargins()
{
  double ratio = (double)_originalImage.height()/_originalImage.width();
  int hMargin = _margin * ratio;
  int vMargin = _margin;

  _margins.setBottom(hMargin);
  _margins.setTop(hMargin);
  _margins.setLeft(vMargin);
  _margins.setRight(vMargin);
  _originalImage.size().rheight()
}

void QImageWidget::setPixmap(QPixmap pixmap)
{
  _originalImage = pixmap;
  updateMargins();
  emit pixmapChanged();
}

void QImageWidget::setMargin(int value)
{
  //устанавливаем границы размером рамки от 2 до 20
  if (value < 2)
    value = 2;
  if (value > 20)
    value = 20;

  //если значение изменилась
  if (value != _margin){
    _margin = value;
    //обновление размеров отступов с учетом соотношений сторон картинки
    updateMargins();
    emit marginChanged();
  }
}

void QImageWidget::mouseDoubleClickEvent(QMouseEvent *event)
{
  //обрабатываем только щелчки левой кнопки
  if (event->button() != Qt::LeftButton)
    return;
  //отображаем диалог выбора файла картинки
  QString filename = QFileDialog::getOpenFileName(this);
  if (filename.isEmpty())
    return;
  //загружаем картинку из выбранного файла
  setPixmap(QPixmap(filename));
}

void QImageWidget::paintEvent(QPaintEvent *)
{
  //создаем объект рисовальщика
  QPainter painter(this);
  painter.setRenderHint(QPainter::Antialiasing);
  //сохраняем конфигурацию
  painter.save();

  //устанавливаем цвет кисти
  painter.setBrush(_backgroundColor);
  //рисуем прямоугольник с размером виджета
  painter.drawRect(rect());

  //возвращаемся к сохраненной конфигурации рисовальщика
  painter.restore();
  //если картинка не загружена
  if (_originalImage.isNull()){
    //рисуем текст
    painter.drawText(rect(), Qt::AlignCenter, _noImageMessage);
  }
  else {
    //получаем область под картинку
    QRect imageRect = actualImageRect();
    //отрисовываем картинку в области за вычетом рамки
    painter.drawPixmap(imageRect.marginsRemoved(_margins), _originalImage);
  }
}





